module Main where



import Control.Monad
import Data.Map (Map)
import qualified Data.Map as Map
import IO
import System (getArgs)
import System.Exit (exitFailure)

import Compile
import Core
import ErrorHandling
import Names
import Parser
import Pretty
import Scope
import Syntax
import HaskellBackend



compileFile :: Scope -> String -> IO Scope
compileFile scope filename = do
	source <- readFile filename
	
	decls <- case parseProgram filename source of
		Left err -> do
			hPutStrLn stderr "Parse error:"
			hPutStrLn stderr (show err)
			exitFailure
		Right syn -> return syn
	
	scope' <- case runErrorHandling (compile scope decls) of
		Left err -> do
			hPutStrLn stderr "Compile error:"
			hPutStrLn stderr err
			exitFailure
		Right d -> return d
	
	return scope'


main = do

	files <- getArgs
	
	-- Compile the files in sequence, each one in the scope produced by the previous one
	
	scope <- foldM compileFile emptyScope files
	
	-- Check to make sure that 'main' is valid
	
	main <- case Map.lookup (Name "main") (scopeValues scope) of
		Nothing -> do
			hPutStrLn stderr "Execution error:"
			hPutStrLn stderr "'main' not in scope."
			exitFailure
		Just v -> return v
	
	when (not (null (pvalueFreeVars main))) $ do
		hPutStrLn stderr "Execution error:"
		hPutStrLn stderr "'main' has a polymorphic type."
		exitFailure
	
	case pvalueType main of
		(Core.FlavorType   -- This is equivalent to "{IO} Unit"
		    (Core.ConType (Name "IO") _)
		    (Core.ConType (Name "Unit") _)
		    ) -> return ()
		
		_ -> do
			hPutStrLn stderr "Execution error:"
			hPutStrLn stderr "main should have type '{IO} Unit'."
			exitFailure
	
	-- Output

	putStr (HaskellBackend.convertProgram scope)

